"""    This file is part of tgenv.

    tgenv is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    tgenv is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with tgenv.  If not, see <https://www.gnu.org/licenses/>."""

import math
import requests

from tqdm import tqdm # type: ignore


def download_asset(url: str, out_file: str, github_token: str):
    """ Starts a download stream and displays a loading bar.

    :param url: The url to download from
    :type url: str
    :param out_file: The file to write
    :type out_file: str
    :param github_token: [description]
    :type github_token: str
    :raises DownloadException: [description]
    """
    if github_token:
        header = {"Authorization": "token" + github_token}
    else:
        header = {}
    download_stream = requests.get(url, stream=True, headers=header)

    size = int(download_stream.headers.get('content-length', 0))
    block_size = 1024
    written = 0
    total = math.ceil(size//block_size)
    stream_iterable = download_stream.iter_content(block_size)
    with open(out_file, 'wb') as asset:
        for data in tqdm(stream_iterable, total=total , unit='KB', unit_scale=1):
            written = written  + len(data)
            asset.write(data)
    if size not in (0, written):
        raise DownloadException("Error while downloading file.")

class DownloadException(Exception):
    """ A generic downloader exception
    """
    def __init__(self, *args):
        Exception.__init__(self, args)



"""    This file is part of tgenv.

    tgenv is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    tgenv is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with tgenv.  If not, see <https://www.gnu.org/licenses/>."""

from github import Github

from .asset_downloader import download_asset

def get_versions(repository, git_token=""):
    """ Gets all published version of a repo
    """
    if git_token:
        github = Github(git_token)
    else:
        github = Github()
    repo = github.get_repo(repository)
    releases = repo.get_releases()
    release_names = []
    for release in releases:
        release_names.append(release.tag_name)
    return release_names

def get_release_asset(repository: str, release_name: str, versions_path: str,git_token=""):
    """ Downloads the specified release asset
    """

    if git_token:
        github = Github(git_token)
    else:
        github = Github()
    repo = github.get_repo(repository)
    release = repo.get_release(release_name)
    assets = release.get_assets()
    for asset in assets:
        if asset.name == "terragrunt_linux_amd64":
            linux_amd64_asset = asset
            break
    url = linux_amd64_asset.browser_download_url
    download_asset(url, versions_path + release_name, git_token)



"""    This file is part of tgenv.

    tgenv is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    tgenv is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with tgenv.  If not, see <https://www.gnu.org/licenses/>."""

import json

from .file_handler import read_versions, version_file_exists, write_version, copy_file

def get_local_versions(filepath: str) -> str:
    """ Returns local versions
    """
    versions = read_versions(filepath)
    versions = json.loads(versions)
    return versions

def add_version(version, filepath, version_file):
    """ Adds a version to the version file

    :param version: The version to add
    :type version: string
    :param filepath: The path to the version
    :type filepath: str
    :param version_file: The file containing the versions
    :type version_file: str
    """
    versions = get_local_versions(version_file)
    versions[version] = filepath
    write_version(version_file, json.dumps(versions))

def has_version(filepath, version):
    """ Checks if version exists
    """
    versions = get_local_versions(filepath)
    try:
        version_file_exists(versions[version])
        return True
    except FileNotFoundError:
        return False
    except KeyError:
        return False

def install_version(target_path, source_path, version):
    """ Installs a version
    """
    copy_file(source_path + version, target_path)



"""    This file is part of tgenv.

    tgenv is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    tgenv is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with tgenv.  If not, see <https://www.gnu.org/licenses/>."""

import os
import sys
import stat
import shutil
import platform

def write_version(file_path: str, versions: str):
    """ Writes local available versions to a file

    :param file_path: Path to the destination file
    :type file_path: str
    :param versions: Content to write
    :type versions: str
    """
    with open(file_path, 'w') as version_file:
        version_file.write(versions)

def read_versions(file_path: str) -> str:
    """ Gets the content of a file
    """
    versions = ""
    with open(file_path, 'r') as version_file:
        versions = version_file.read()
    return versions

def version_file_exists(file_path: str) -> bool:
    """ Checks if the file exists
    """
    if os.path.isfile(file_path):
        return True
    raise FileNotFoundError

def copy_file(source: str, dest: str) -> bool:
    """ Copies a file from dest to source and gives executable rights
    """
    abs_path =  os.path.expanduser(dest)
    if not os.path.isfile(abs_path):
        with open(abs_path, "w") as file:
            file.write("")
    shutil.copyfile(source, abs_path)
    perm_state = os.stat(abs_path)
    os.chmod(abs_path, perm_state.st_mode | stat.S_IXUSR)

def check_files(path: str, version_file: str):
    """ Checks if the file structure is present and creates it if not

    :param path: Path to the dest folder
    :type path: str
    """
    versions_folder = os.path.expanduser(path + "versions")
    versions_file = os.path.expanduser(path + version_file)

    if not os.path.exists(versions_folder):
        os.makedirs(versions_folder)

    if not os.path.exists(versions_file):
        with open(versions_file, 'w') as ver_file:
            ver_file.write("{}")

def check_for_wsl() -> int:
    """ Checks if running on wsl
    """
    retval = 0
    if os.path.exists("/mnt/c/Windows") or sys.platform == "Windows":
        retval = 1
    elif platform.system() == "Darwin":
        retval = 2
    return retval
